(ns com.aurellem.run.image
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run util title save-corruption
                          bootstrap-0 bootstrap-1))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])
  (:import java.io.File))

;; want to display an image onto the screen.
;; probably will be the six ponies, possibly with scrolling. 

;; probably don't need hi-color mode since the images shuld be
;; simple. 

;; use background tiles?  they provide greater color depth than
;; sprites, and can still be scrolled, so why not? 


;; First of all, RGB colors in an image are not the same as those in a
;; GameBoy, so I need to convert them.  Fortunately, this code is
;; already written for me in this C-code from the public domain
;; hi-color converter by Glen Cook, Jeff Frohwein, and Rob Jones.

;; the code snipped itself is by Brett Bibby and is translated here
;; from C into clojure.


;; This section of code is used to convert an RGB (pc) triplet into
;; a RGB (gameboy) triplet. This section of code was kindly donated
;; by Brett Bibby (GameBrains).

;; BYTE intensity[32] = {
;;   0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x5e, 0x6c, 0x7a, 0x88, 0x94,
;;   0xa0, 0xae, 0xb7, 0xbf, 0xc6, 0xce, 0xd3, 0xd9, 0xdf, 0xe3, 0xe7,
;;   0xeb, 0xef, 0xf3, 0xf6, 0xf9, 0xfb, 0xfd, 0xfe, 0xff, 0xff };

;; unsigned char influence[3][3] = 
;;   {
;;     {16,4,4},
;;     {8,16,8},
;;     {0,8,16}
;;   };

;; RGBQUAD translate(BYTE rgb[3])
;; {
;;   RGBQUAD color;
;;   BYTE	tmp[3];
;;   BYTE	m[3][3];
;;   BYTE	i,j;

;;   for (i=0;i<3;i++)
;;     for (j=0;j<3;j++)
;;       m[i][j] = (intensity[rgb[i]>>3]*influence[i][j]) >> 5;

;;   for (i=0;i<3;i++)
;;     {
;;       if (m[0][i]>m[1][i])
;; 	{
;; 	  j=m[0][i]; 
;; 	  m[0][i]=m[1][i]; 
;; 	  m[1][i]=j;
;; 	}

;;       if (m[1][i]>m[2][i])
;; 	{
;; 	  j=m[1][i]; 
;; 	  m[1][i]=m[2][i]; 
;; 	  m[2][i]=j;
;; 	}

;;       if (m[0][i]>m[1][i])
;; 	{
;; 	  j=m[0][i]; 
;; 	  m[0][i]=m[1][i]; 
;; 	  m[1][i]=j;
;; 	}

;;       tmp[i]=(((m[0][i]+m[1][i]*2+m[2][i]*4)*5) >> 4)+32;
;;     }

;;   color.rgbRed	= tmp[0];
;;   color.rgbGreen	= tmp[1];
;;   color.rgbBlue	= tmp[2];

;;   return color;
;; }


                  

(def image-program-target 0xB000)


(def display-width 160)
(def display-height 144)



;{:r :g :b }

(def character-data     0x8000)
(def character-data-end 0x97FF)




(def BG-data-1 0x9800)

(def BG-data-2 0x9C00)

(def OAM 0xFE00)


(def bg-pallet-select 0xFF68)
(def bg-pallet-data   0xFF69)

(def obj-palette-select 0xFF6A)
(def obj-palette-data   0xFF6B)



(def video-bank-select-register 0xFF4F)



(defn gb-rgb->bits [r g b]
  (assert (< 0 r 32))
  (assert (< 0 g 32))
  (assert (< 0 b 32))
  [(bit-and
    0xFF
    (+
     r
     (bit-shift-left g 5)))

   (+
    (bit-shift-right g 3)
    (bit-shift-left  b 2))])
      
      
   

  )
 
(defn display-one-color
  "Displayes a single color onto the gameboy screen. input rgb in
   gameboy rgb."
  [r g b]
  




  )